from __future__ import annotations

from dataclasses import dataclass
from th2_cli.utils import is_ip

NODE_DEFAULT_VALUE = HOST_DEFAULT_VALUE = ''


@dataclass
class KubernetesConfigSection:
    pvs_node: str = NODE_DEFAULT_VALUE
    host: str = HOST_DEFAULT_VALUE

    @property
    def hostname(self) -> str | None:
        if is_ip(self.host):
            return None
        return self.host

    def pvs_node_is_default(self) -> bool:
        return self.pvs_node == NODE_DEFAULT_VALUE

    def host_is_default(self) -> bool:
        return self.host == HOST_DEFAULT_VALUE



from dataclasses import dataclass

HOST_DEFAULT_VALUE = '127.0.0.1'
DC_DEFAULT_VALUE = 'datacenter1'


@dataclass
class CassandraConfigSection:
    host: str = HOST_DEFAULT_VALUE
    datacenter: str = DC_DEFAULT_VALUE

    def host_is_default(self) -> bool:
        return self.host == HOST_DEFAULT_VALUE

    def datacenter_is_default(self) -> bool:
        return self.datacenter == DC_DEFAULT_VALUE



from th2_cli.utils.infra import pre_delete_warning, get_infra_mgr_config, delete_namespace, delete_pv
from th2_cli.utils import read_value, print_info, print_used_value, print_error
from th2_cli.utils.kubernetes import connect, get_namespaces, get_pvs
from halo import Halo
import time


def delete():
    pre_delete_warning()
    print_info('Connecting to the cluster...')
    k8s_client, k8s_core = connect()
    prefix = 'th2-'
    try:
        prefix = get_infra_mgr_config(k8s_core)['kubernetes']['namespacePrefix']
    except:
        print_error('Unable to get namespace prefix from config')
    th2_namespaces = list(filter(
        lambda ns: ns.startswith(prefix) or ns in ['service', 'monitoring'],
        get_namespaces(k8s_core)))
    print_info('Deleting th2 namespaces...')
    for ns in th2_namespaces:
        print(f'  Deleting {ns} namespace...')
        delete_namespace(k8s_core, ns)
    spinner = Halo(text='Waiting until all th2 namespaces are terminated...', spinner='dots')
    spinner.start()
    while len(th2_namespaces) != 0:
        th2_namespaces = list(filter(
            lambda ns: ns.startswith(prefix) or ns in ['service', 'monitoring'],
            get_namespaces(k8s_core)))
        time.sleep(1)
    spinner.stop()
    print_info('Deleting th2 PersistentVolumes...')
    for pv in get_pvs(k8s_core):
        print(f'  Deleting {pv} PersistentVolume...')
        delete_pv(k8s_core, pv)
    print_info('th2 is deleted from your Kubernetes cluster')




import deepmerge

from th2_cli.utils import read_value, is_ip, write_file, print_info, print_used_value, yes_no_input, check_file, \
    enter_to_continue
from th2_cli.utils.kubernetes import connect, get_cluster_host, create_secret
from th2_cli.utils.cassandra import choose_datacenter
from th2_cli.utils.crypto import generate_ssh_keys
from th2_cli.utils.helm.charts_installer import ChartsInstaller
from th2_cli.utils.infra import install_flannel, create_namespace, choose_node, \
    apply_yaml, pv_folders_warning, preinstall_warning
from th2_cli.utils.install_config import InstallConfig
from simple_term_menu import TerminalMenu
from th2_cli.templates.install import InstallTemplates

TH2_VERSION = '1.7.3'


def install():
    preinstall_warning()
    print_info('Connecting to the cluster...')
    k8s_client, k8s_core = connect()
    if check_file('th2-cli-install-config.yaml'):
        if yes_no_input('th2 installation config is detected. Do you want to load configuration?', True):
            install_config = InstallConfig.from_yaml_file('th2-cli-install-config.yaml')
        else:
            install_config = InstallConfig()
    else:
        install_config = InstallConfig()
    print_info("Preparing Kubernetes cluster...")
    # Install flannel
    install_flannel(k8s_client)
    # Create namespaces
    create_namespace(k8s_core, 'monitoring')
    create_namespace(k8s_core, 'service')
    # Apply PV's
    print('Please choose node for storing PersistentVolumes')
    if install_config.kubernetes.pvs_node_is_default():
        install_config.kubernetes.pvs_node = choose_node(k8s_core)
    print_used_value('Node for PersistentVolumes storage', install_config.kubernetes.pvs_node)
    pv_folders_warning()
    print_info('Creating PV\'s and PVC\'s...')
    apply_yaml(k8s_client, InstallTemplates.pvs(node_name=install_config.kubernetes.pvs_node), 'pvs.yaml')
    apply_yaml(k8s_client, InstallTemplates.pvcs(), 'pvcs.yaml')
    # Get information about Kubernetes cluster
    if install_config.kubernetes.host_is_default():
        install_config.kubernetes.host = get_cluster_host(k8s_client)
    print_used_value('Cluster host', install_config.kubernetes.host)
    if is_ip(install_config.kubernetes.host):
        install_config.kubernetes.host = read_value('Enter Kubernetes cluster hostname, if it exist.', 'hostname') \
                                         or install_config.kubernetes.host
        print_used_value('Cluster hostname', install_config.kubernetes.hostname or '')
    # Get information about Cassandra database
    if install_config.cassandra.host_is_default():
        install_config.cassandra.host = read_value('Enter hostname of Cassandra to access it from the '
                                                   'Kubernetes cluster', 'host', '127.0.0.1')
    print_used_value('Cassandra host', install_config.cassandra.host)
    if install_config.cassandra.datacenter_is_default():
        dc_input_index = TerminalMenu(
            [
                'Enter datacenter manually',
                'Connect to Cassandra and choose available datacenter (Admin credentials required)'
            ],
            title='How do you want to specify Cassandra datacenter?'
        ).show()
        if dc_input_index == 0:
            install_config.cassandra.datacenter = read_value('Enter Cassandra datacenter name.', 'datacenter',
                                                             'datacenter1')
        else:
            install_config.cassandra.datacenter = choose_datacenter(install_config.cassandra.host)
    print_used_value('Cassandra datacenter', install_config.cassandra.datacenter)
    # Get information about infra-schema
    if install_config.infra_mgr.git.repository_is_default():
        install_config.infra_mgr.git.repository = read_value('Enter link to your infra-schema', 'link')
    print_used_value('th2-infra-schema link', install_config.infra_mgr.git.repository)
    if install_config.infra_mgr.git.repository.startswith('https://'):
        if install_config.infra_mgr.git.http_auth_username_is_default() or install_config.infra_mgr.git.http_auth_password_is_default():
            print_info('th2 will be authenticated in git by Personal Access Token (PAT)')
            token = read_value('Enter PAT for your infra-schema', 'PAT')
            install_config.infra_mgr.git.http_auth_username = token
            install_config.infra_mgr.git.http_auth_password = token
        print_used_value('Personal Access Token', install_config.infra_mgr.git.http_auth_username)
        create_secret(k8s_core, 'infra-mgr', namespace='service', string_data={'infra-mgr': 'infra-mgr'})
    else:
        print_info('th2 will be authenticated in git by SSH key')
        private_key, public_key = generate_ssh_keys()
        write_file('infra-mgr-rsa.key', private_key)
        write_file('infra-mgr-rsa.key.pub', public_key)
        create_secret(k8s_core, 'infra-mgr', namespace='service', string_data={'infra-mgr': private_key})
    if check_file('secrets.yaml'):
        if yes_no_input('"secrets.yaml" file is detected. Would you like to rewrite it with template?'):
            InstallTemplates.create_secrets_template()
            print_info('"secrets.yaml" template is created.')
    else:
        InstallTemplates.create_secrets_template()
        print_info('"secrets.yaml" template is created.')
    print('Fill "secrets.yaml" with relevant credentials')
    enter_to_continue()
    # Deploy infrastructure
    print_info('Deploying monitoring infrastructure...')
    charts_installer = ChartsInstaller(namespace='monitoring')
    charts_installer.add_helm_release('prometheus-community', 'kube-prometheus-stack',
                                      'https://prometheus-community.github.io/helm-charts', '15.0.0',
                                      InstallTemplates.prometheus_operator_values(
                                          hosts=install_config.kubernetes.hostname or ''))
    charts_installer.add_helm_release('grafana', 'loki-stack',
                                      'https://grafana.github.io/helm-charts', '2.4.1',
                                      InstallTemplates.loki_values())
    charts_installer.install_charts()
    print_info('Deploying service infrastructure...')
    charts_installer = ChartsInstaller(namespace='service')
    charts_installer.add_helm_release('ingress-nginx', 'ingress-nginx',
                                      'https://kubernetes.github.io/ingress-nginx', '3.31.0',
                                      InstallTemplates.ingress_values())
    th2_values: dict = deepmerge.always_merger.merge(
        InstallTemplates.service_values(
                                              schema_link=install_config.infra_mgr.git.repository,
                                              git_username=install_config.infra_mgr.git.http_auth_username,
                                              git_password=install_config.infra_mgr.git.http_auth_password,
                                              cluster_host=install_config.kubernetes.host,
                                              cluster_hostname=install_config.kubernetes.hostname or '',
                                              cassandra_host=install_config.cassandra.host,
                                              cassandra_datacenter=install_config.cassandra.datacenter),
        InstallTemplates.get_secrets())
    charts_installer.add_helm_release('th2', 'th2',
                                      'https://th2-net.github.io', '1.7.3',
                                      th2_values)
    charts_installer.install_charts()
    print_info(f'th2 {TH2_VERSION} is installed')
    if yes_no_input('Would you like to save th2 config in the current directory?', True):
        write_file('th2-cli-install-config.yaml', install_config.to_yaml())



from th2_cli.cli.install import install
from th2_cli.cli.delete import delete
from th2_cli.cli.status import status
from th2_cli.cli.version import get_th2cli_info
from th2_cli.cli.mgr import InfraMgr

import signal


def handler(signum, frame):
    print("Ctrl-C was pressed. Exiting ")
    exit(1)


class Th2Cli:

    mgr = InfraMgr()

    def version(self):
        get_th2cli_info()

    def install(self):
        signal.signal(signal.SIGINT, handler)
        install()

    def delete(self):
        signal.signal(signal.SIGINT, handler)
        delete()

    def status(self):
        status()



from dataclasses import dataclass
from dataclass_wizard import YAMLWizard
from th2_cli.utils.install_config.kubernetes import KubernetesConfigSection
from th2_cli.utils.install_config.cassandra import CassandraConfigSection
from th2_cli.utils.install_config.infra_mgr import InfraMgrConfigSection


@dataclass
class InstallConfig(YAMLWizard):
    kubernetes: KubernetesConfigSection = KubernetesConfigSection()
    cassandra: CassandraConfigSection = CassandraConfigSection()
    infra_mgr: InfraMgrConfigSection = InfraMgrConfigSection()



yaml = '''
infraMgr:
  git:
    repository: <repository>
    httpAuthUsername: <username>
    httpAuthPassword: <password>

infraOperator:
  config:
    k8sUrl: "<host>"
    
infraEditor:
  image:
    repository: ghcr.io/th2-net/th2-infra-editor
    tag: 1.0.65

ingress:
  host: &host <hostname>

dashboard:
  metrics-server:
    enabled: false
  ingress:
    paths:
      - /dashboard($|/.*)

rabbitmq:
  prometheus:
    operator:
      enabled: true
  persistentVolume:
    enabled: true
    storageClass: local-storage
    size: 10Gi
  ingress:
    enabled: true
    hostName: <hostname>

cassandra:
  internal: false
  host: <cassandra-host>
  cluster:
    datacenter: <cassandra-dc>
'''



from dataclasses import dataclass


REPO_DEFAULT_VALUE = \
    USERNAME_DEFAULT_VALUE = \
    PASSWORD_DEFAULT_VALUE = ''


@dataclass
class GitSection:
    repository: str = REPO_DEFAULT_VALUE
    http_auth_username: str = USERNAME_DEFAULT_VALUE
    http_auth_password: str = PASSWORD_DEFAULT_VALUE

    def repository_is_default(self) -> bool:
        return self.repository == REPO_DEFAULT_VALUE

    def http_auth_username_is_default(self) -> bool:
        return self.http_auth_username == USERNAME_DEFAULT_VALUE

    def http_auth_password_is_default(self) -> bool:
        return self.http_auth_password == PASSWORD_DEFAULT_VALUE



from kubernetes.client import ApiClient, CoreV1Api, V1Namespace
from kubernetes import client, config
from urllib.parse import urlparse
from typing import Tuple, List
from th2_cli.utils import print_error


def connect() -> Tuple[ApiClient, CoreV1Api]:
    try:
        config.load_kube_config()
    except:
        print_error('kube config is not valid or not presented')
        exit(1)
    k8s_client = client.ApiClient()
    k8s_core = client.CoreV1Api()
    try:
        k8s_core.list_node()
    except:
        print_error('Kubernetes cluster is unavailable')
        exit(1)
    return k8s_client, k8s_core


def create_namespace_object(name: str) -> V1Namespace:
    return V1Namespace(metadata=client.V1ObjectMeta(name=name))


def get_pods(k8s_core: CoreV1Api, namespace: str = 'default') -> List[str]:
    pods = list(
        map(lambda item: item.metadata.name, k8s_core.list_namespaced_pod(namespace).items)
    )
    return pods


def get_pod_status(k8s_core: CoreV1Api, pod: str, namespace: str = 'default') -> str:
    p = k8s_core.read_namespaced_pod(pod, namespace)
    return p.status.phase


def get_pod_logs(k8s_core: CoreV1Api, pod: str, namespace: str = 'default') -> str:
    logs = k8s_core.read_namespaced_pod_log(pod, namespace)
    return logs


def get_namespaces(k8s_core: CoreV1Api) -> List[str]:
    namespaces = list(
        map(lambda item: item.metadata.name, k8s_core.list_namespace().items)
    )
    return namespaces


def get_namespace_status(k8s_core: CoreV1Api, name: str) -> str:
    namespace = k8s_core.read_namespace(name)
    return namespace.status.phase


def get_pvs(k8s_core: CoreV1Api) -> List[str]:
    pvs = list(
        map(lambda item: item.metadata.name, k8s_core.list_persistent_volume().items)
    )
    return pvs


def get_nodes(k8s_core: CoreV1Api) -> List[str]:
    nodes = list(
        map(lambda item: item.metadata.name, k8s_core.list_node().items)
    )
    return nodes


def get_cluster_host(k8s_client: ApiClient) -> str:
    url = k8s_client.configuration.host
    return urlparse(url).netloc.split(':')[0]


def create_secret(k8s_core: CoreV1Api, name: str, data: dict = None, string_data: dict = None,
                  namespace: str = 'service'):
    try:
        secret = client.V1Secret(
            metadata=client.V1ObjectMeta(name=name),
            data=data,
            string_data=string_data
        )
        k8s_core.create_namespaced_secret(namespace=namespace,
                                          body=secret)
    except:
        print_error(f'Failed to create "{name}" secret')


def delete_secret(k8s_core: CoreV1Api, name: str, namespace: str = 'service'):
    try:
        k8s_core.delete_namespaced_secret(name, namespace)
    except:
        print_error(f'Failed to delete "{name}" secret')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['th2_cli',
 'th2_cli.cli',
 'th2_cli.cli.mgr',
 'th2_cli.templates',
 'th2_cli.templates.install',
 'th2_cli.templates.install.values',
 'th2_cli.utils',
 'th2_cli.utils.helm',
 'th2_cli.utils.install_config',
 'th2_cli.utils.install_config.infra_mgr']

package_data = \
{'': ['*']}

install_requires = \
['avionix>=0.4.5,<0.5.0',
 'cassandra-driver>=3.25.0,<4.0.0',
 'colorama>=0.4.5,<0.5.0',
 'cryptography>=37.0.4,<38.0.0',
 'dataclass-wizard[yaml]>=0.22.1,<0.23.0',
 'deepmerge>=1.0.1,<2.0.0',
 'fire>=0.4.0,<0.5.0',
 'halo>=0.0.31,<0.0.32',
 'kubernetes>=24.2.0,<25.0.0',
 'requests>=2.28.1,<3.0.0',
 'simple-term-menu>=1.5.0,<2.0.0',
 'urllib3>=1.26.12,<2.0.0']

entry_points = \
{'console_scripts': ['th2 = th2_cli:cli']}

setup_kwargs = {
    'name': 'th2-cli',
    'version': '1.7.3',
    'description': '👨\u200d💻 CLI for managing th2 infrastructure in Kubernetes cluster',
    'long_description': 'Works with th2 1.7.3\n\n## Using\n\nInstall:\n\n```commandline\npip install th2-cli\n```\n\nCheck version of CLI:\n\n```commandline\nth2 version\n```\n\nOutput example:\n\n```commandline\n$ th2 version\nth2 CLI v1.7.3\nSource repository: https://github.com/d0rich/th2-cli\n```\n\n### Install th2\n\nIf you already have configurations in `th2-cli-install-config.yaml` or `secrets.yaml`, it will be convenient to run process from the directory with these files.\nIn other case CLI will create these config files during installation.\n\n```commandline\nth2 install\n```\n\n### Delete th2\n\n```commandline\nth2 delete\n```\n\n### Update th2\n\n```commandline\nth2 delete\n```\n\nWait until all required namespaces are terminated.\n\n```commandline\nth2 install\n```\n\n### Get th2 status\n\nDisplay information about all th2-related namespaces in Kubernetes.\n\n```commandline\nth2 status\n```\n\n### infra-mgr\n\nDisplay status of infra-mgr pod:\n\n```commandline\nth2 mgr status\n```\n\nDisplay last logs of infra-mgr pod:\n\n```commandline\nth2 mgr logs\n```\n\n## Configurations templates\n\n### th2-cli-install-config.yaml\n\n```yaml\ncassandra:\n  datacenter: datacenter1\n  host: host.minikube.internal\ninfra-mgr:\n  git:\n    http-auth-password: pat_token\n    http-auth-username: pat_token\n    repository: https://github.com/schema/repository\nkubernetes:\n  host: 192.168.49.2\n  pvs-node: minikube\n```\n\n### secrets.yaml\n\n```yaml\n# required only for images from a private registry, will be attached as the first PullSecret to deployments\n#productRegistry:\n#  username: user\n#  password: password\n#  name: private-registry-1.example.com # core components registry\n\n# required only for images from a private registry, will be attached as the second PullSecret to deployments\n#solutionRegistry:\n#  username: user\n#  password: password\n#  name: private-registry-2.example.com # components registry\n\n# required only for images from a private registry, will be attached as the third PullSecret to deployments\n#proprietaryRegistry:\n#  username: user\n#  password: password\n#  name: private-registry-3.example.com # components registry\n\ncassandra:\n# set credentials for the existing Cassandra cluster\n  dbUser:\n    user: cassandra\n    password: cassandra\n\nrabbitmq:\n# set admin user credentials, it will be created during deployment\n  rabbitmqUsername: th2\n  rabbitmqPassword: rab-pass\n  # must be random string\n  rabbitmqErlangCookie: cookie\n```\n\n## Development\n\n```\npoetry install\npoetry shell\n```\n\n```commandline\nth2 install\n```',
    'author': 'Nikolay Dorofeev',
    'author_email': 'dorich2000@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/d0rich/th2-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from th2_cli.utils.kubernetes import connect, get_namespaces, get_namespace_status
from th2_cli.utils import print_error


def status():
    k8s_client, k8s_core = connect()
    namespaces = get_namespaces(k8s_core)
    namespaces = list(filter(lambda ns: ns in ['monitoring', 'service'] or 'th2' in ns, namespaces))
    if len(namespaces):
        for ns in namespaces:
            print(f'{ns} - {get_namespace_status(k8s_core, ns)}')
    else:
        print_error('th2 is not installed')



from th2_cli.utils.kubernetes import get_pods, get_pod_status, connect
from th2_cli.utils import print_error


def status():
    k8s_client, k8s_core = connect()
    pods = get_pods(k8s_core, 'service')
    mgr_pod = next((pod for pod in pods if pod.startswith('infra-mgr')), None)
    if not mgr_pod:
        print_error('infra-mgr does not exist')
    else:
        print(get_pod_status(k8s_core, mgr_pod, 'service'))
